/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */
// CustomList.cpp : implementation file
//
#include "stdafx.h"
#include "ui.h"
#include "CustomList.h"
#include "uiDlg.h"
#include "HwndFunctions.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "notify/timed_msgbox.h"

#include "initguid.h"
#include "imgguids.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCustomList

CCustomList::CCustomList()
{

    brushHollow.CreateStockObject(HOLLOW_BRUSH);

    nItemWidth   = 400;
    nItemHeight  = 30;
    locked = false;

    // Main logo members
    mainLogo = NULL;
    mainLogoCounter = 0;
    isMainLogoAnimated = false;
}

CCustomList::~CCustomList()
{
    // release all list item.
    while( pItemList.GetCount() )
    {
        CExtItem* pItem =  pItemList.GetAt(0);
        delete pItem;
        pItemList.RemoveAt(0,1);
    }

    if (mainLogo) {
    	mainLogo->Release();
        mainLogo = NULL;
        CoUninitialize();
    }

    DestroyWindow();
}

BEGIN_MESSAGE_MAP(CCustomList, CListCtrl)
	//{{AFX_MSG_MAP(CCustomList)
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
    ON_WM_LBUTTONDOWN()
	ON_WM_CREATE()
    ON_MESSAGE(WM_SETFONT, OnSetFont)
    ON_WM_MEASUREITEM_REFLECT( )
    //ON_WM_DRAWITEM( )
    ON_WM_TIMER()
    ON_WM_ERASEBKGND()
    ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCustomList message handlers

void CCustomList::Init()
{
    // Insert at least one column to the list control
    VERIFY(InsertColumn(0, _T(""), LVCFMT_LEFT) == 0);
	SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);

    // Initialize the main logo only if it's a portal build.
    ClientSettings* cs = getRegConfig();
    if(cs->getIsPortal()) {
        initMainLogo(false);
    }
}

void CCustomList::PreSubclassWindow()
{
	Init();
	CListCtrl::PreSubclassWindow();
}

int CCustomList::GetCurSelItem() const
{
    ASSERT(::IsWindow(m_hWnd));

    // Return the index of the item with the focus
    return GetNextItem(-1, LVNI_FOCUSED);
}

BOOL CCustomList::SetCurSelItem(int nIndex)
{
    ASSERT(::IsWindow(m_hWnd));

    // Set the focus to the item with the specified index
    return SetItemState(nIndex, LVIS_FOCUSED, LVIS_FOCUSED);
}



#if defined(_DEVICE_RESOLUTION_AWARE)
/**
 * Returns the offset of status labels from the left edge.
 * Actually we use 1/3 of the main screen width.
 */
int CCustomList::GetTextOffSet()
{
    // Get client area          *** TODO: use a class variable to store client rect ***
    CRect rect;
    GetClientRect(&rect);

    //wchar_t tmp[100];
    //wsprintf(tmp, L"Width = %d", rect.Width());
    //AfxMessageBox(tmp);

    // minimum status with = 128px (otherwise we won't read the whole labels...)
    int statusWidth = max((int)(rect.Width() * 0.74), 128);

    // 26% of the width for the source title ("Contacts")
    // 74% of the width for the source status ("Not Synchronized")
    return (rect.Width() - statusWidth);
}
#endif


void CCustomList::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
    ASSERT(::IsWindow(m_hWnd));
    ASSERT(lpDrawItemStruct != 0);
    if( (! lpDrawItemStruct) || (! lpDrawItemStruct->hDC) ){
        return;
    }
    CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);
    ASSERT(pDC != NULL);
    if(!pDC){
        return;
    }

    // fix width
    lpDrawItemStruct->rcItem.right = nItemWidth;

    int nItem = lpDrawItemStruct->itemID;

    int nSavedDC = pDC->SaveDC();
    if(nSavedDC == 0)
        return;

    // a memory DC, we first draw into memory and then copy the content to the device screen
    CDC tempdc; tempdc.CreateCompatibleDC(pDC);

    // Get item image and state info
    LV_ITEM lvi;
    lvi.mask = LVIF_IMAGE | LVIF_STATE;
    lvi.iItem = nItem;
    lvi.iSubItem = 0;
    lvi.stateMask = 0xFFFF;     // get all state flags
    GetItem(&lvi);

    // Should the item be highlighted
    BOOL bHighlight =  lvi.state & LVIS_FOCUSED;

    CRect rectFull(lpDrawItemStruct->rcItem);
    // icon and text rect: get square dimension from panes height
    int iconSize = rectFull.Height();
    CRect rectIcon(rectFull.left,  rectFull.top, rectFull.left+iconSize, rectFull.top+iconSize);
    CRect rectText(rectIcon.right,rectFull.top,rectFull.right,rectFull.bottom);

    CExtItem* pItem = NULL;
    if(lpDrawItemStruct->itemID > (UINT)pItemList.GetUpperBound())
        return;

    pItem = pItemList.GetAt(lpDrawItemStruct->itemID);

    if(! pItem){
        LOG.error("CustomList: item is NULL.");
        return;
    }

    tempdc.SetBkMode(TRANSPARENT);

    // we initialize the memory DC
    CRect rectClient;
    CBitmap bmpTemp;

    GetClientRect(&rectClient);
    bmpTemp.CreateCompatibleBitmap(pDC,rectClient.Width(), rectClient.Height());

    HBITMAP pOldBitmap = (HBITMAP) tempdc.SelectObject(bmpTemp);

    pDC->SetBkMode(TRANSPARENT);

    if(bHighlight){
        // item has been selected - draw selected frame
        CRect rect(rectFull);

        tempdc.Rectangle(rect);

        rect.DeflateRect(1,1,1,1);

        tempdc.FillSolidRect(rect, LIST_COLOR_SELECTED_ITEM);	// fill item frame

        // draw icon
        if(pItem->getIcon()){
            // Repositioning knowing the icons are 22x22
            // (-2px to leave some space before the label, minimum offset = 0)
            int offset = max((rectIcon.Width() - 22)/2, 0);
            tempdc.DrawIcon(rectIcon.left+offset-2, rectIcon.top+offset, pItem->getIcon());
        }

        if(pItem->isEnabled()){
            tempdc.SetTextColor(LIST_COLOR_TEXT);
        }
        else{
            tempdc.SetTextColor(GetSysColor(COLOR_GRAYTEXT));
        }

        // we select default font from device CDC, not memory CDC
        CFont* pOldPDCFont = pDC->SelectObject(&fontBold);
        CFont* pOldtempDCFont = tempdc.SelectObject(&fontBold);

        rect.CopyRect(rectText);
        rect.DeflateRect(2,2);
        #if defined(WIN32_PLATFORM_PSPC)
            rect.OffsetRect(-6,6);
        #else
            rect.OffsetRect(-6,4);
        #endif

        if( pItem->getName() != ""){
            tempdc.DrawText(pItem->getName(), lstrlen(pItem->getName()),
                rect,DT_LEFT | DT_SINGLELINE);
        }

        tempdc.SelectObject(pOldPDCFont);
        pDC->SelectObject(pOldPDCFont);
        pOldtempDCFont->DeleteObject();

        #if defined(WIN32_PLATFORM_PSPC)
            rect.OffsetRect(GetTextOffSet(), 0);
        #else
            rect.OffsetRect(GetTextOffSet(), 1);
        #endif

        if( pItem->getText() != ""){
            tempdc.DrawText(pItem->getText(), lstrlen(pItem->getText()),
                rect,DT_LEFT | DT_SINGLELINE);
        }
    }
    else{
        //  NOT HIGHLIGHTED
        // item has been selected or draw all item
        // fill background for setted background color
        CRect rect(rectFull);

        tempdc.FillSolidRect(rect, LIST_COLOR_BACKGROUND);	// fill item frame

        // draw icon
        if(pItem->getIcon()) {
            // Repositioning knowing the icons are 22x22
            // (-2px to leave some space before the label, minimum offset = 0)
            int offset = max((rectIcon.Width() - 22)/2, 0);
            tempdc.DrawIcon(rectIcon.left+offset-2, rectIcon.top+offset, pItem->getIcon());
        }

        if(pItem->isEnabled()){
            tempdc.SetTextColor(LIST_COLOR_TEXT);
        }
        else{
            tempdc.SetTextColor(GetSysColor(COLOR_GRAYTEXT));
        }

        CFont* pOldPDCFont = pDC->SelectObject(&fontBold);
        CFont* pOldtempdcFont = tempdc.SelectObject(&fontBold);

        rect.CopyRect(rectText);
        rect.DeflateRect(2,2);
        #if defined(WIN32_PLATFORM_PSPC)
            rect.OffsetRect(-6,6);
        #else
            rect.OffsetRect(-6,4);
        #endif

        if( pItem->getName() != ""){
            tempdc.DrawText(pItem->getName(),lstrlen(pItem->getName()),
                rect,DT_LEFT | DT_SINGLELINE);
        }

        tempdc.SelectObject(pOldPDCFont);
        pDC->SelectObject(pOldPDCFont);
        pOldtempdcFont->DeleteObject();

        #if defined(WIN32_PLATFORM_PSPC)
            rect.OffsetRect(GetTextOffSet(), 0);
        #else
            rect.OffsetRect(GetTextOffSet(), 1);
        #endif

        if( pItem->getText() != ""){
            tempdc.DrawText(pItem->getText(), lstrlen(pItem->getText()),
                rect,DT_LEFT | DT_SINGLELINE);
        }

    }

    // we finished drawing into memory, now we copy it to the device screen
    CRect rectBlt(lpDrawItemStruct->rcItem);
    pDC->BitBlt(rectBlt.TopLeft().x, rectBlt.TopLeft().y, rectBlt.Width(),
            rectBlt.Height(), &tempdc, rectBlt.TopLeft().x,rectBlt.TopLeft().y,SRCCOPY);

    tempdc.SelectObject(pOldBitmap);
    bmpTemp.DeleteObject();

    tempdc.DeleteDC();
    //pDC->RestoreDC(nSavedDC);
}

void CCustomList::RepaintSelectedItems()
{
    CRect rcBounds, rcLabel;

    // Invalidate focused item so it can repaint
    int nItem = GetNextItem(-1, LVNI_FOCUSED);

    if(nItem != -1)
    {
        GetItemRect(nItem, rcBounds, LVIR_BOUNDS);
        GetItemRect(nItem, rcLabel, LVIR_LABEL);
        rcBounds.left = rcLabel.left;

        InvalidateRect(rcBounds, FALSE);
    }

    // Invalidate selected items depending on LVS_SHOWSELALWAYS
    if(!(GetStyle() & LVS_SHOWSELALWAYS))
    {
        for(nItem = GetNextItem(-1, LVNI_SELECTED);
        nItem != -1; nItem = GetNextItem(nItem, LVNI_SELECTED))
        {
            GetItemRect(nItem, rcBounds, LVIR_BOUNDS);
            GetItemRect(nItem, rcLabel, LVIR_LABEL);
            rcBounds.left = rcLabel.left;

            InvalidateRect(rcBounds, FALSE);
        }
    }
#ifdef _DEBUG
    else
        ASSERT(FALSE);
#endif

    UpdateWindow();
}

void CCustomList::OnSetFocus(CWnd* pOldWnd)
{
	CListCtrl::OnSetFocus(pOldWnd);

    // check if we are getting focus from label edit box
    if (pOldWnd != NULL && pOldWnd->GetParent() == this){
        return;
    }

    // repaint items that should change appearance
    if ((GetStyle() & LVS_TYPEMASK) == LVS_REPORT){
        RepaintSelectedItems();
    }

    #ifdef _DEBUG
    else
        ASSERT(FALSE);
    #endif
}

void CCustomList::OnKillFocus(CWnd* pNewWnd)
{
	CListCtrl::OnKillFocus(pNewWnd);

    // check if we are losing focus to label edit box
    if (pNewWnd != NULL && pNewWnd->GetParent() == this){
        return;
    }

    // repaint items that should change appearance
    if ((GetStyle() & LVS_TYPEMASK) == LVS_REPORT)
        RepaintSelectedItems();
#ifdef _DEBUG
    else
        ASSERT(FALSE);
#endif
}

void CCustomList::OnLButtonDown(UINT nFlags, CPoint point){
    // if the list is locked we ignore the message
    if(locked)
        return;

    // try to see if this item is selected for the 2nd time
    int oldSel, newSel;
    oldSel = GetCurSelItem();
    Default(); // call default message handler
    newSel = GetCurSelItem();

    CRect rect; GetItemRect(newSel, &rect, LVIR_BOUNDS);

    if( (rect.PtInRect(point)) &&  (newSel == oldSel) ){
        CWnd* wndMain = AfxGetMainWnd();
        if(wndMain)
            ((CuiDlg*)wndMain)->SelectSource(newSel);
    }

    // clears non-item part of list
    Invalidate();
}

void CCustomList::setText(int id, CString text ){
    CString s1;
    s1+= "("; s1+=text;
    s1+=")";

    CExtItem* pItem = getItem(id);
    if(pItem){
        pItem->setText(s1);
        InvalidatePane(id);
    }
}

CString CCustomList::getText(int id){
    CExtItem* pItem = getItem(id);

    if(pItem){
        return pItem->getText();
    }

    return CString("");
}

void CCustomList::enableItem( int id, bool enable ){
    CExtItem* pItem = getItem(id);
    pItem->setEnabled(enable);
}

void CCustomList::setState( int id, int state ){
    CExtItem* item = getItem(id);
    if(item){
        item->setItemState(state);
    }
}

void CCustomList::InvalidatePane( int id ){
    CRect rect;
    GetItemRect(idToIndex(id),rect,LVIR_BOUNDS);
    InvalidateRect(rect);
}

void CCustomList::setIcon( int id, HICON hIcon ){
    CExtItem* pItem = getItem(id);
    if(pItem){
        pItem->setIcon(hIcon);
    }
}

void CCustomList::startAnim(int id){
    this->SetTimer(id + ANIM_TIMER_OFFSET, ANIM_ICON_DELAY, NULL);
}

void CCustomList::stopAnim(int id){
    CExtItem* pItem = getItem(id);

    this->KillTimer(id + ANIM_TIMER_OFFSET);
    resetSourceIcon(id);
}

bool CCustomList::isEnabledItem( int id ){
    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->isEnabled();
    }
    return false;
}

int CCustomList::getState( int id ){
    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->getItemState();
    }

    return 1; // state_ok default
}

void CCustomList::setStateToAll( int state, bool onlyEnabled /*= false*/ )
{
    for (int i=0;i<=pItemList.GetUpperBound(); i++){
        CExtItem* pItem = pItemList.GetAt(i);
        if(pItem){
            if(onlyEnabled) {
                if(pItem->isEnabled()){
                    pItem->setItemState(state);
                }
            }
            else{
                pItem->setItemState(state);
            }
        }
    }
}


void CCustomList::MeasureItem(LPMEASUREITEMSTRUCT lpMIS)
{
    lpMIS->itemHeight  = nItemHeight;
    lpMIS->itemWidth   = nItemWidth;
}


LRESULT CCustomList::OnSetFont(WPARAM wParam, LPARAM)
{
    LRESULT res =  Default();

    CRect rc;
    GetWindowRect( &rc );

    WINDOWPOS wp;
    wp.hwnd = m_hWnd;
    wp.cx = rc.Width();
    wp.cy = rc.Height();
    wp.flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER;
    SendMessage( WM_WINDOWPOSCHANGED, 0, (LPARAM)&wp );

    return res;
}


void CCustomList::addItem(int id, CString lpszItemName,HICON hIcon)
{
    CExtItem* pItem = new CExtItem(id, lpszItemName,hIcon);
    pItemList.Add(pItem);

    InsertItem(GetItemCount(), lpszItemName);
}

CExtItem* CCustomList::getItem( int id ){
    int i=0;
    while(i<pItemList.GetCount()){
        if(pItemList.GetAt(i)->getId() == id){
            return pItemList.GetAt(i);
        }
        i++;
    }

    return NULL; // should not get here
}


void CCustomList::OnTimer(UINT_PTR nIDEvent ){

    if (nIDEvent == MAIN_LOGO_ID) {
        // It's the main logo, manage it separately.
        animateMainLogo();
        return;
    }

    int sourceId = nIDEvent - ANIM_TIMER_OFFSET;
    if(!doesIdExist(sourceId)) {
        return;
    }
    CExtItem* pItem = getItem(sourceId);
    if(pItem == NULL) {
        return;
    }


    int counter = pItem->getCounterAnim();
    if(counter == 4){
        counter = 0;
    }

    switch(counter){
    case 0:
        this->setIcon(sourceId, AfxGetApp()->LoadIcon(IDI_ARROWS22A));
        break;
    case 1:
        this->setIcon(sourceId, AfxGetApp()->LoadIcon(IDI_ARROWS22B));
        break;
    case 2:
        this->setIcon(sourceId, AfxGetApp()->LoadIcon(IDI_ARROWS22C));
        break;
    case 3:
        this->setIcon(sourceId, AfxGetApp()->LoadIcon(IDI_ARROWS22D));
        break;
    };

    counter++;

    pItem->setCounterAnim(counter);
    InvalidatePane(sourceId);
}

void CCustomList::resetSourceIcon( int id ){
    int iconId = -1;
    bool isItemEnabled = isEnabledItem(id);

    switch(id){
        case SOURCE_MAIL:
            iconId = isItemEnabled ? IDI_MAIL : IDI_MAIL_DIM; break;
        case SOURCE_CONTACTS:
            iconId = isItemEnabled ? IDI_CONTACTS : IDI_CONTACTS_DIM; break;
        case SOURCE_CALENDAR:
            iconId = isItemEnabled ? IDI_CALENDAR : IDI_CALENDAR_DIM; break;
        case SOURCE_TASKS:
            iconId = isItemEnabled ? IDI_TASKS : IDI_TASKS_DIM; break;
        case SOURCE_BRIEFCASE:
            iconId = isItemEnabled ? IDI_BRIEFCASE : IDI_BRIEFCASE_DIM; break;

        #if defined(WIN32_PLATFORM_PSPC)
            case SOURCE_NOTES:
                iconId = isItemEnabled ? IDI_NOTES : IDI_NOTES_DIM; break;
        #endif
    }

    if(iconId != -1)
        setIcon(id, AfxGetApp()->LoadIcon(iconId));
}

int CCustomList::indexToId( int index ){
    if(index < 0){
        return 0;
        // TODO: signal error
    }
    CExtItem* item = pItemList.GetAt(index);
    if(item){
        return item->getId();
    }

    return 0;
}

int CCustomList::idToIndex( int id ){
    CExtItem* pItem;
    int i = 0;
    int index = 0;

    while(i <= pItemList.GetUpperBound() ){
        pItem = pItemList.GetAt(i);
        if(pItem){
            if(pItem->getId() == id){
                index = i;
            }
        }
       i++;
    }

    return index;
}


void CCustomList::resetSourceIcons(){
    for(int i=0;i<GetItemCount();i++)
        resetSourceIcon(indexToId(i));
}



void CCustomList::setIsSyncing( int id, bool value ){
    CExtItem* item = getItem(id);
    if(item){
        item->setIsItemSyncing(value);
    }
}

bool CCustomList::getIsSyncing( int id ){
    CExtItem* item = getItem(id);
    if(item){
        return item->getIsItemSyncing();
    }

    return false;
}

void CCustomList::OnPaint(){
    // call default handler to redraw items if needed
    Default();

    // now we repaint area in which there are no items
    CPaintDC dc(this);
    CRect rect, rectLastItem;
    // get client area
    GetClientRect(&rect);

    // get last item area
    int count = GetItemCount();
    GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);

    // fill the area below the last item
    
    dc.FillSolidRect(0, rectLastItem.BottomRight().y, rect.Width(),
        rect.Height()-rectLastItem.BottomRight().y, LIST_COLOR_BACKGROUND);
    
    // Redraw the logo on the right 
    // We need to calculate again the main logo position, as it could
    // be changed from Init (it happens on square smartphone)
    calculateMainLogoPosition();
    updateMainLogoImage();
}

BOOL CCustomList::OnEraseBkgnd( CDC* pDC ){
    // we disable automatic redrawing of list background

    // now we repaint area in which there are no items
    CRect rect, rectLastItem;
    CDC tempdc; tempdc.CreateCompatibleDC(pDC);
    CBitmap bmpTemp;

    // get client area
    GetClientRect(&rect);

    int nSavedDC = pDC->SaveDC();

    // get last item area
    int count = GetItemCount();
    GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);

    CRect rectDraw(0, rectLastItem.BottomRight().y, rect.Width(), rect.Height()-rectLastItem.BottomRight().y);
    tempdc.SetBkMode(TRANSPARENT);

    bmpTemp.CreateCompatibleBitmap(&tempdc, rectDraw.Width(), rectDraw.Height());
    HBITMAP pOldBitmap = (HBITMAP) tempdc.SelectObject(bmpTemp);

    // fill the area below the last item
    tempdc.FillSolidRect(0, rectLastItem.BottomRight().y, rect.Width(),
        rect.Height()-rectLastItem.BottomRight().y, LIST_COLOR_BACKGROUND);

    // bitblt into pDC
    pDC->BitBlt(rectDraw.TopLeft().x, rectDraw.TopLeft().y, rectDraw.Width(),
        rectDraw.Height(), &tempdc, rectDraw.TopLeft().x,rectDraw.TopLeft().y,SRCCOPY);

    tempdc.SelectObject(pOldBitmap);
    bmpTemp.DeleteObject();

    tempdc.DeleteDC();

    return TRUE;
}

BOOL CCustomList::GetItemRect( int nItem,LPRECT lpRect, UINT nCode ) const{
    CListCtrl::GetItemRect(nItem, lpRect, nCode);
    lpRect->right = nItemWidth;

    return 0;
}

bool CCustomList::doesIdExist( int id ){
    CExtItem* pItem;
    int i = 0;
    int index = -1;

    while(i <= pItemList.GetUpperBound() ){
        pItem = pItemList.GetAt(i);
        if(pItem){
            if(pItem->getId() == id){
                index = i;
            }
        }
        i++;
    }

    if(index == -1){
        return false;
    }
    else{
        return true;
    }
}




/**
 * Animates the main logo.
 * The internal counter is updated, and then the image is refreshed.
 */
void CCustomList::animateMainLogo() {
    
    if (!mainLogo) {
        return;
    }

    mainLogoCounter++;
    if(mainLogoCounter == MAIN_LOGO_NUM_FRAMES){
        mainLogoCounter = 0;
    }

    //Invalidate();
    InvalidateRect(&mainLogoRect);
    updateMainLogoImage();
}


/**
 * Updates the main logo image:
 * - calculates the image section to redraw (depends on 'mainLogoCounter')
 * - draws the image to the screen
 */
void CCustomList::updateMainLogoImage() {

    if (!mainLogo) {
        return;
    }

    // Get the effective image section to be drawn
    CPaintDC dc(this);
    int xOffset = 0;
    if (isMainLogoAnimated) {
        xOffset = mainLogoCounter * mainLogoSize.cx;
    }
    RECT offsetRc = {xOffset, 0, mainLogoSize.cx + xOffset, mainLogoSize.cy};

    mainLogo->Draw(dc.GetSafeHdc(), &mainLogoRect, &offsetRc);
}


/**
 * Initializes the main logo image.
 * The image 'mainLogo' will be loaded only if it's a portal build AND 
 * the file 'MAIN_LOGO_NAME' is found under the install dir.
 * If it's not a portal build or file not found, no image will be loaded and
 * 'mainLogo' member is NULL.
 * If a previous 'mainLogo' was loaded, we release it before loading the new logo
 * (this method is called when starting/stopping the animation)
 *
 * @param animatedImage  if true, loads the animated logo
 *                       if false, loads the static logo
 * 
 * - gets the full path of the image file
 * - loads the image file and stores it to the internal member 'mainLogo'
 * - calculates the rect where image will be drawn, and real logo dimension
 */
void CCustomList::initMainLogo(bool animatedImage) {

    if (mainLogo) {
    	mainLogo->Release();
        mainLogo = NULL;
        CoUninitialize();
    }

    ClientSettings* cs = getRegConfig();
    if(cs->getIsPortal() == false) {
        return;
    }

    // Static logo has 1 frame, animated logo has 20 frames
    string imageName;
    int numFrames = 1;
    if (animatedImage) {
        imageName = MAIN_LOGO_NAME;
        numFrames = MAIN_LOGO_NUM_FRAMES;
    }
    else {
        imageName = MAIN_LOGO_STATIC_NAME;
        numFrames = 1;
    }

    // Get the image full path
    string imagePath = cs->getAppPath();
    imagePath += "\\";
    imagePath += imageName;
    WCHAR* wImagePath = toWideChar(imagePath.c_str());


    // Create the imaging factory.
    IImagingFactory *pImgFactory = NULL;
    CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (SUCCEEDED(CoCreateInstance (CLSID_ImagingFactory, NULL, CLSCTX_INPROC_SERVER, IID_IImagingFactory, (void **)&pImgFactory))) {        
        pImgFactory->CreateImageFromFile(wImagePath, &mainLogo);
        pImgFactory->Release();
    }
    if (wImagePath) { 
        delete [] wImagePath; wImagePath = NULL; 
    }

    if (mainLogo) {
        // Image found: calculate position and size
        calculateMainLogoPosition();

        // Set the logo dimension: mainLogoSize
        SIZE imageSize;
        mainLogo->GetPhysicalDimension(&imageSize);
        mainLogoSize.cx = imageSize.cx / numFrames;
        mainLogoSize.cy = imageSize.cy;
    }
    else {
        // No image found!
        CoUninitialize();
        mainLogoRect.bottom = mainLogoRect.top = 0;
        mainLogoRect.left = mainLogoRect.right = 0;
        mainLogoSize.cx = mainLogoSize.cy = 0;
    }
}



/**
 * Gets the rectangle for the main logo (mainLogoRect) and the
 * effective logo dimension (mainLogoSize) from the current 
 * main screen size.
 * They're always the same, so this method is called at the
 * beginning or if the screen size changes.
 */
void CCustomList::calculateMainLogoPosition() {

    if (!mainLogo) {
        return;
    }

    CPaintDC dc(this);
    CRect rect;
    GetClientRect(&rect);
    mainLogoRect.left   = rect.Width()  - MAIN_LOGO_LEFT_CORNER;
    mainLogoRect.right  = rect.Width()  - MAIN_LOGO_RIGHT_CORNER;
    mainLogoRect.top    = rect.Height() - MAIN_LOGO_UPPER_CORNER;    
    mainLogoRect.bottom = rect.Height() - MAIN_LOGO_LOWER_CORNER;

    // Correction of logo dimensions: if logo overlaps the last item pane,
    // we decrease the logo size to fill only the white area at the bottom.
    int count = GetItemCount();
    if (count > 0) {
        CRect rectLastItem;
        GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);
        if (mainLogoRect.top < rectLastItem.bottom) {
            mainLogoRect.left += (rectLastItem.bottom - mainLogoRect.top);
            mainLogoRect.top = rectLastItem.bottom;
        }
    }

    // Avoid drawing logo if it's too small...
    if ((mainLogoRect.bottom - mainLogoRect.top) < 10) {
        mainLogoRect.bottom = mainLogoRect.top = 0;
        mainLogoRect.left = mainLogoRect.right = 0;
    }
}





/**
 * Starts the timer for the main logo animation.
 * 'OnTimer(MAIN_LOGO_ID)' will be called every MAIN_LOGO_DELAY milliseconds.
 */
void CCustomList::startMainLogo() {
    if (!mainLogo) {
        return;
    }
    if (MAIN_LOGO_USE_ANIMATION) {
        initMainLogo(true);                 // Loads the animated logo
        SetTimer(MAIN_LOGO_ID, MAIN_LOGO_DELAY, NULL);
        isMainLogoAnimated = true;
    }
}


/**
 * Kills the timer for the main logo animation.
 */
void CCustomList::stopMainLogo() {
    if (!mainLogo) {
        return;
    }
    if (MAIN_LOGO_USE_ANIMATION) {
        if (KillTimer(MAIN_LOGO_ID)) {
            isMainLogoAnimated = false;
            initMainLogo(false);            // Loads the static logo

            // Force 1st redraw
            InvalidateRect(&mainLogoRect);
            updateMainLogoImage();

        }
    }
}

