/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#ifndef INCL_CTP_PARAM
    #define INCL_CTP_PARAM
/** @cond DEV */
    
    #include "stdio.h"     
    #include <stdio.h>
    #include <windows.h>   
    #include "base/Log.h"
       
    /**
    * Params
    */
    #define P_DEVID       0x01
    #define P_USERNAME    0x02
    #define P_CRED        0x03
    #define P_FROM        0x04
    #define P_TO          0x05
    #define P_NONCE       0x06
    #define P_SAN         0x07   
    #define P_SLEEP       0x09   
    
    /**
    * The single CTP Param class that represents the parameter that are exchanged
    * from server to client
    */

    class CTPParam {

        private:
            
            /**
            * The code of the param. It could be one of the #define in the class
            */
            char  paramCode;

            /**
            * The length of the value
            */
            int   valueLength;

            /**
            * The byte array of the value exchanged
            */
            void* value;

             /**
            * Set the value length
            */
            void setValueLength(int v) { valueLength = v; }


        public:

            /**
            * Constructor
            */
            CTPParam();

            /**
            * Desctructor
            */
            ~CTPParam();
            
            /**
            * Copy constructor
            */
            CTPParam(const CTPParam& cp);

            /**
            * Set the param code
            */
            void setParamCode(char v) { paramCode = v; }            
            
            /**
            * Return the param code
            */
            char getParamCode() const { return paramCode; }
                       
            /**
            * Get the value length
            */
            int getValueLength() const { return valueLength; }
                                   
            /**
            * Set the value: in creates an internal byte array
            * of size length. It set the variable valueLength.
            * If succeded it return the pointer to the just byte array allocated
            * If failed it returns NULL
            */
            void* setValue(const void* v, long length);

            /**
            * Return the pointer to the internal byte array. The caller
            * MUST NOT free the pointer
            */
            
            void* CTPParam::getValue() const { return value; }

    };
    
    
/** @endcond */
#endif
