/*
* Copyright (C) 2003-2007 Funambol, Inc
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
* PURPOSE.  See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
* 02111-1307  USA
*/

// PIMSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "PIMSettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"
#include "ExploreFolders.h"

#include "pim/clientsettings.h"
#include "pim/Maincpp.h"

// CPIMSettings dialog

IMPLEMENT_DYNAMIC(CPIMSettings, CDialog)

CPIMSettings::CPIMSettings(CWnd* pParent /*=NULL*/)
                           : CDialog(CPIMSettings::IDD, pParent) {
}

CPIMSettings::~CPIMSettings() {
}


void CPIMSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTPIM_SPIN1, spinSyncType );
#endif
    DDX_Control(pDX, IDC_PIM_SYNCTYPE, lstSyncType );
 //   DDX_Control(pDX, IDC_PIM_LINK_ADVANCED, linkAdvanced);
#if defined(WIN32_PLATFORM_PSPC)
    DDX_Control(pDX, IDC_NOTES_LINK_CHOOSE, linkNotesChoose);
#endif
    DDX_Control(pDX, IDC_BRIEFCASE_LINK_CHOOSE, linkBriefcaseChoose);
}


BEGIN_MESSAGE_MAP(CPIMSettings, CDialog)
   // ON_BN_CLICKED(IDC_PIM_LINK_ADVANCED, &CPIMSettings::OnBnClickedPIMLinkAdvanced)
#if defined(WIN32_PLATFORM_PSPC)
    ON_BN_CLICKED(IDC_NOTES_LINK_CHOOSE, &CPIMSettings::OnBnClickedNotesLinkChoose)
#endif
    ON_BN_CLICKED(IDC_BRIEFCASE_LINK_CHOOSE, &CPIMSettings::OnBnClickedBriefcaseLinkChoose)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


void CPIMSettings::OnBnClickedNotesLinkChoose(){
 
  #if defined(WIN32_PLATFORM_PSPC)
    CExploreFolders wndExplore;
    wndExplore.setIsNotes(true);
    wndExplore.setFolder(folder);

    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE)
    {
        // user has chosen a new folder
        CString folderToSync = wndExplore.getFolder();
        CString s1("");
        //s1.LoadString(IDS_FOLDER_TO_SYNC_NOTE);
        s1+=folderToSync;
        this->folder = folderToSync;
        SetDlgItemText(IDC_NOTES_STATIC_FOLDER, s1);
    }
  #endif
}

BOOL CPIMSettings::OnInitDialog(){
    // SET window title
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    // Link for advanced settings is actually hidden.
//    linkAdvanced.ShowWindow(FALSE);

#ifdef WIN32_PLATFORM_WFSP
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);
#endif

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
#if defined WIN32_PLATFORM_WFSP
#if (_WIN32_WCE < 0x500)
    dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
#endif
#endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    };

    // read string resources
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE2);	lstSyncType.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE3);	lstSyncType.AddString(s1);
    //s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_SETTINGS);  linkAdvanced.setText(s1);

    LoadStringResources();

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->SetFont(&fontBold);

    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS); 
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif


    // Read from the registry, get values from 'contact' syncsource
    // as they should be all the same!
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
    if (!ssconf) {
        TRACE(_T("Source 'contact' not found in registry!"));
        return FALSE;
    }
    CString s3; int pos=0;

    pos = SynctypeReg2UI(ssconf->getSync());
    lstSyncType.SetCurSel(pos);

    // check if another sync is in progress
    if(checkStartSync() != 0){
        lstSyncType.EnableWindow(FALSE);
        #if defined(WIN32_PLATFORM_WFSP)
        spinSyncType.EnableWindow(FALSE);
        #endif
    }

    if (!getRegConfig()->getIsPortal()) {
        CString foldertosync;
#if defined(WIN32_PLATFORM_PSPC)
        foldertosync.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES);
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FOLDER_TO_SYNC_NOTE);
        foldertosync += s1;
        SetDlgItemText(IDC_NOTES_STATIC_FOLDERTOSYNC,foldertosync);
        GetDlgItem(IDC_NOTES_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);

        // read from the registry 
        WindowsSyncSourceConfig* ssconfnotes = getRegConfig()->getWinSyncSourceConfig("note");
        pos=0;

        pos = SynctypeReg2UI(ssconfnotes->getSync());
        lstSyncType.SetCurSel(pos);

#if defined(WIN32_PLATFORM_WFSP)
        // override back key
        LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
            SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
        ::SendMessage(dlgCommandBar3, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
#endif

        s3 = ssconfnotes->getSyncDir().c_str();
        if (s3 == "") {
            // default
            LPTSTR dir= new WCHAR[256];
            SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
            s3 = CString(dir);
            s3+="\\notes";
            delete [] dir;
        }

        folder = s3;
        SetDlgItemText(IDC_NOTES_STATIC_FOLDER,s3);
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHOOSE);
        linkNotesChoose.setText(s1);

#endif
        //briefcase

        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);


        foldertosync.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE);
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FOLDER_TO_SYNC);
        foldertosync += s1;
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC,foldertosync);

        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHOOSE);
        linkBriefcaseChoose.setText(s1);

        WindowsSyncSourceConfig* ssconfbriefcase = getRegConfig()->getWinSyncSourceConfig("briefcase");
        pos = SynctypeReg2UI(ssconfbriefcase->getSync());
        lstSyncType.SetCurSel(pos);

        s3 = ssconfbriefcase->getSyncDir().c_str();
        folder = s3;

        if (s3 == "") {
            LPTSTR dir = new  WCHAR[512];
            SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
            wstring s(dir);
            delete [] dir;
            wstring brief = s; brief += L"\\briefcase";
            if (existsDirectory((wchar_t*)brief.c_str())) {
                s3 = brief.c_str();
            } else {
                s3 = s.c_str();
            }
        }

        folder = s3;
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER,folder);
        s3.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE_CHOOSE_FOLDER);

    }else{
#if defined(WIN32_PLATFORM_PSPC)
        GetDlgItem(IDC_NOTES_STATIC_FOLDERTOSYNC)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_NOTES_STATIC_FOLDER)->ShowWindow(SW_HIDE);
        linkNotesChoose.ShowWindow(SW_HIDE);
#endif

        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDER)->ShowWindow(SW_HIDE);
        linkBriefcaseChoose.ShowWindow(SW_HIDE);
    }


    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    lstSyncType.SetFocus();
    return FALSE;
}

void CPIMSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }

    int pos = lstSyncType.GetCurSel();
    const char* syncType = SynctypeUI2Reg(pos);

    // Save value in registry for contacts
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
    if (!ssconf) {
        TRACE(_T("Source 'contact' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
    }

    // Save value in registry for calendar
    ssconf = getRegConfig()->getWinSyncSourceConfig("calendar");
    if (!ssconf) {
        TRACE(_T("Source 'calendar' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_CALENDAR);
    }

    // Save value in registry for tasks
    ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    if (!ssconf) {
        TRACE(_T("Source 'task' not found in registry!"));
        return;
    }
    if(strcmp(syncType, ssconf->getSync())){
        ssconf->setSync(syncType);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
    }
    getRegConfig()->saveDirty();

    CDialog::OnOK();
}



void CPIMSettings::LoadStringResources(){
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCTYPE); 
    SetDlgItemText(IDC_SETTPIM_STATIC_SYNCTYPE, s1);
}

void CPIMSettings::OnBnClickedBriefcaseLinkChoose()
{
    CExploreFolders wndExplore;

    #if defined(WIN32_PLATFORM_PSPC)
        wndExplore.setIsNotes(false);
    #endif

    wndExplore.setFolder(folder);

    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE){
        //wstring s = wndExplore.getFolder();
        folder = wndExplore.getFolder();
        //SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, s.c_str());
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, folder);
    }

}

void CPIMSettings::OnBnClickedPIMLinkAdvanced() {
    // Not yet implemented, advanced link is not used.
    //CPIMSettingsAdvanced wndContactsAdvanced;
    //INT_PTR result = wndContactsAdvanced.DoModal();
}


// capture pressing of 'Advanced' link
BOOL CPIMSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
     /*       if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedPIMLinkAdvanced();
            }*/
            if(GetFocus()->GetSafeHwnd() == linkNotesChoose.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedNotesLinkChoose();
            }
            if(GetFocus()->GetSafeHwnd() == linkBriefcaseChoose.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedBriefcaseLinkChoose();
            }
        }
    }

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

HBRUSH CPIMSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}