/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// SyncSettingsAdvanced.cpp : implementation file
//

#include "stdafx.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "processUtils.h"
#include "SyncMethod.h"
#include "PushSettings.h"
// CSyncMethod dialog

IMPLEMENT_DYNAMIC(CSyncMethod, CDialog)

char minutes[9][5] = {"15","30","60","120","240","360","480","720","1440"};


int getPollingPosition(string value) {
    if (value == "15")   return 0;
    if (value == "30")   return 1;
    if (value == "60")   return 2;
    if (value == "120")  return 3;
    if (value == "240")  return 4;
    if (value == "360")  return 5;
    if (value == "480")  return 6;
    if (value == "720")  return 7;
    if (value == "1440") return 8;
    return 0; //default
}

int getSyncMethodPosition(CString value) 
{
    CString s1;
    HINSTANCE localResource = getLocalizationUtils()->getLocaleResource();

    s1.LoadString(localResource, IDS_PUSH_SERVICE);
    if (value == s1) { return 0; }
    s1.LoadString(localResource, IDS_MANUAL_SYNC);
    if (value == s1) { return 1; }
    s1.LoadString(localResource, IDS_SCHEDULED_SYNC);
    if (value == s1) { return 2; }
    return 0;   // default, should not happen...
}


CSyncMethod::CSyncMethod(CWnd* pParent /*=NULL*/)
	: CDialog(CSyncMethod::IDD, pParent)
{
    // Just to store the local resources handle...
    localResource = getLocalizationUtils()->getLocaleResource();
}

CSyncMethod::~CSyncMethod()
{
}

void CSyncMethod::DoDataExchange(CDataExchange* pDX)
{

    DDX_Control(pDX, IDC_SCHEDULER_SYNCMETHOD, lstSyncMethod);
    DDX_Control(pDX, IDC_SCHEDULER_MINUTES, lstPolling);
    #ifdef WIN32_PLATFORM_WFSP
        DDX_Control(pDX, IDC_SCHEDULER_SPIN_SYNCMETHOD, spinSyncMethod);
        DDX_Control(pDX, IDC_SCHEDULER_SPIN, spinPolling);
    #endif
	CDialog::DoDataExchange(pDX);
}

BOOL CSyncMethod::OnInitDialog(){
    CString s1;
    s1.LoadString(localResource, IDS_SYNC_METHOD);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    #ifdef WIN32_PLATFORM_WFSP
    spinSyncMethod.SetRange(2,0);  spinSyncMethod.SetBuddy(&lstSyncMethod);
        spinPolling.SetRange(8,0); spinPolling.SetBuddy(&lstPolling);
    #endif

    // set menubar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
        #endif
    #endif
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
       TRACE(_T("Cannot create command bar!"));
       return FALSE;
    }

    // load string resources
    s1.LoadString(localResource, IDS_SCHEDULER);           SetDlgItemText(IDC_SYNC_STATIC_SCHEDULER, s1);
    s1.LoadString(localResource, IDS_INTERVAL);            SetDlgItemText(IDC_SYNC_STATIC_INTERVAL, s1);
 
    // Add strings to lstSyncMethod list
    lstSyncMethod.ResetContent();
    s1.LoadString(localResource, IDS_PUSH_SERVICE);     lstSyncMethod.AddString(s1);
    s1.LoadString(localResource, IDS_MANUAL_SYNC);      lstSyncMethod.AddString(s1);
    s1.LoadString(localResource, IDS_SCHEDULED_SYNC);   lstSyncMethod.AddString(s1);

    // Add strings to lstPolling list
    lstPolling.ResetContent();
    s1.LoadString(localResource, IDS_POLLING15);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING30);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING1H);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING2H);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING4H);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING6H);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING8H);    lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING12H);   lstPolling.AddString(s1);
    s1.LoadString(localResource, IDS_POLLING1D);    lstPolling.AddString(s1);


    // create font
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SYNC_STATIC_SCHEDULER)->SetFont(&fontBold);
    GetDlgItem(IDC_SYNC_STATIC_INTERVAL)->SetFont(&fontBold);
    
     // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(localResource, IDS_SYNC_METHOD); 
        SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif


    // read settings from Registry
    ClientSettings* cs = getRegConfig();

 

    // Load default polling time from registry
    initialSchedulerSelection = lstPolling.GetCurSel();
    lstPolling.SetCurSel(getPollingPosition(cs->getPolling()));
    lstPolling.UpdateWindow();


    // Read syncMethod from registry
    s1 = getActiveSyncMethod();
    int syncMethodPos = getSyncMethodPosition(s1);
    lstSyncMethod.SetCurSel(syncMethodPos);

    switch (syncMethodPos) {
        case 0:                         // 0 = Push service
        case 1:                         // 1 = Manual sync
            // Scheduled is disabled
            lstPolling.EnableWindow(FALSE);
            GetDlgItem(IDC_SYNC_STATIC_INTERVAL)->EnableWindow(FALSE);
            break;
        case 2:                         // 2 = Scheduled sync
            // Scheduled enabled
            lstPolling.EnableWindow(TRUE);
            GetDlgItem(IDC_SYNC_STATIC_INTERVAL)->EnableWindow(TRUE);
            break;
        default:
            break;
    }

    lstPolling.UpdateWindow();
    lstSyncMethod.UpdateWindow();
    lstSyncMethod.SetFocus();

    // Store initial positions, for checks on saving 
    initialSchedulerSelection = lstPolling.GetCurSel();
    initialSyncMethodSelection = lstSyncMethod.GetCurSel();

    return FALSE;
}

/**
 * Save settings to registry
 */
afx_msg void CSyncMethod::OnOK(){

    ClientSettings* cs = getRegConfig();
    BOOL dirtyFlag = FALSE;

    if(checkStartSync()){
        CDialog::OnOK();
    }


    // Read current selections and initial selections
    int syncMethodPos = lstSyncMethod.GetCurSel();
    int schedulerPos  = lstPolling.GetCurSel();
    bool startPush = false;
    bool stopPush  = false;
    if (syncMethodPos != initialSyncMethodSelection) {
        if (syncMethodPos == 0) {
            startPush = true;
    }
        else if (initialSyncMethodSelection == 0) {
            stopPush = true;
        }
    }


    if (startPush) {
        //
        // Start all push services: SMS + STP with fallback to CTP
        //
        startPushService();
        dirtyFlag = TRUE;
    }

    else if (stopPush) {
        //
        // Stop all push services
        //
        stopPushService();
        dirtyFlag = TRUE;
    }


    if (syncMethodPos == 2) {
        //
        // Scheduled sync is selected:
        // set the schedule value for 'polling' and for all sources
        //
        string schedMinutes = minutes[schedulerPos];
        cs->setPolling(schedMinutes);
        cs->getWinSyncSourceConfig("mail"     )->setSchedule(schedMinutes);
        cs->getWinSyncSourceConfig("contact"  )->setSchedule(schedMinutes);
        cs->getWinSyncSourceConfig("calendar" )->setSchedule(schedMinutes);
        cs->getWinSyncSourceConfig("task"     )->setSchedule(schedMinutes);
        // Notes and briefcase are disabled
        //cs->getWinSyncSourceConfig("note"     )->setSchedule(schedMinutes);
        //cs->getWinSyncSourceConfig("briefcase")->setSchedule(schedMinutes);
        startProgram(_T("startsync.exe"), _T("schedule"));
        dirtyFlag = TRUE;
    }
    else if (initialSyncMethodSelection == 2) {
        //
        // Stop scheduling
        //
        cs->setPolling("0");
        cs->getWinSyncSourceConfig("mail"     )->setSchedule("0");
        cs->getWinSyncSourceConfig("contact"  )->setSchedule("0");
        cs->getWinSyncSourceConfig("calendar" )->setSchedule("0");
        cs->getWinSyncSourceConfig("task"     )->setSchedule("0");
        // Notes and briefcase are disabled
        //cs->getWinSyncSourceConfig("note"     )->setSchedule("0");
        //cs->getWinSyncSourceConfig("briefcase")->setSchedule("0");
        startProgram(_T("startsync.exe"), _T("schedule"));
        dirtyFlag = TRUE;
    }

    


    if(dirtyFlag){
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_ALL);
        getRegConfig()->setDirty(CS_DIRTY_ROOT_CONFIG);
        getRegConfig()->saveDirty();
    }
    
    CDialog::OnOK();
}

BEGIN_MESSAGE_MAP(CSyncMethod, CDialog)
    ON_WM_CTLCOLOR()
    ON_CBN_SELCHANGE(IDC_SCHEDULER_SYNCMETHOD, &CSyncMethod::OnCbnSelchangeSchedulerSyncmethod)
END_MESSAGE_MAP()


// CSyncMethod message handlers
HBRUSH CSyncMethod::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}


/**
 * Actions executed wehen lstSyncMethod list changes.
 */
void CSyncMethod::OnCbnSelchangeSchedulerSyncmethod()
{
    int syncMethodPos = lstSyncMethod.GetCurSel();
    switch (syncMethodPos) {
        case 0:                         // 0 = Push service
        case 1:                         // 1 = Manual sync
            // Scheduled is disabled
            lstPolling.EnableWindow(FALSE);
            GetDlgItem(IDC_SYNC_STATIC_INTERVAL)->EnableWindow(FALSE);
            break;
        case 2:                         // 2 = Scheduled sync
            // Scheduled enabled
            lstPolling.EnableWindow(TRUE);
            GetDlgItem(IDC_SYNC_STATIC_INTERVAL)->EnableWindow(TRUE);
            break;
        default:
            break;
    }
}



/**
 * Read sync method from config, returns one of:
 * "Push sevice" , "Manual Synchronization", "Scheduled Synchronization"
 * Strings are loaded from local resources.
 * Only the 'contact' synsource is analyzed for the scheduled info.
 */
CString getActiveSyncMethod() {

    CString s1;
    HINSTANCE localResource = getLocalizationUtils()->getLocaleResource();
    ClientSettings* cs = getRegConfig();
    int pushType = 0;
    
    if (cs->getPush().size() > 0) {
        pushType = atoi(cs->getPush().c_str());
    }

    if (pushType > 0) {
        s1.LoadString(localResource, IDS_PUSH_SERVICE);
    }
    else {
        int schedule = 0;
        if (cs->getWinSyncSourceConfig("contact")->getSchedule().size() > 0) {
            schedule = atoi(cs->getWinSyncSourceConfig("contact")->getSchedule().c_str());
        }
        if (schedule > 0)  { s1.LoadString(localResource, IDS_SCHEDULED_SYNC); }
        else               { s1.LoadString(localResource, IDS_MANUAL_SYNC);    }
    }

    return s1;
}
